@@ -10,6 +10,7 @@ import os
 from warnings import warn
 
 import numpy as np
+import h5py
 from skimage.measure import block_reduce
 from skimage.util import crop
 
@@ -17,11 +18,8 @@ from .df_utils import dm4reader
 from .df_utils.dm_utils import parse_dm4_parms, read_dm3
 from ...core.io.translator import Translator, generate_dummy_main_parms
 from ...core.io.write_utils import Dimension, calc_chunks
-from ...core.io.hdf_utils import get_h5_obj_refs, link_as_main
+from ...core.io.hdf_utils import create_indexed_group, write_main_dataset, write_simple_attrs
 from ...core.io.image import read_image
-from ..hdf_writer import HDFwriter
-from ..virtual_data import VirtualGroup, VirtualDataset
-from ..write_utils import build_ind_val_dsets
 
 
 class OneViewTranslator(Translator):
@@ -34,7 +32,7 @@ class OneViewTranslator(Translator):
 
         self.rebin = False
         self.bin_factor = 1
-        self.hdf = None
+        self.h5_f = None
         self.binning_func = self.__no_bin
         self.bin_func = None
         self.h5_main = None
@@ -89,13 +87,11 @@ class OneViewTranslator(Translator):
 
         """
         # Open the hdf5 file and delete any contents
-        try:
-            hdf = HDFwriter(h5_path)
-            hdf.clear()
-        except:
-            raise
+        if os.path.exists(h5_path):
+            os.remove(h5_path)
+        h5_f = h5py.File(h5_path, 'w')
 
-        self.hdf = hdf
+        self.h5_f = h5_f
         self.crop_method = crop_method
         self.crop_ammount = crop_ammount
 
@@ -103,6 +99,7 @@ class OneViewTranslator(Translator):
         Get the list of all files with the .tif extension and
         the number of files in the list
         '''
+        image_path = os.path.abspath(image_path)
         root_file_list, file_list = self._parse_file_path(image_path)
 
         size, image_parms = self._getimageparms(file_list[0])
@@ -154,7 +151,7 @@ class OneViewTranslator(Translator):
         self._read_data(file_list[start_image:start_image + num_files],
                         h5_main, h5_mean_spec, h5_ronch, image_path)
 
-        self.hdf.close()
+        self.h5_f.close()
 
         return
 
@@ -179,51 +176,30 @@ class OneViewTranslator(Translator):
         Create the Measurement and Channel Groups to hold the
         image Datasets
         '''
-        root_grp = VirtualGroup('/')
+        meas_grp = create_indexed_group(self.h5_f, 'Measurement')
 
-        meas_grp = VirtualGroup('Measurement_')
-
-        chan_grp = VirtualGroup('Channel_')
-        root_grp.add_children([meas_grp])
-        meas_grp.add_children([chan_grp])
+        chan_grp = create_indexed_group(meas_grp, 'Channel')
 
         '''
         Set the Measurement Group attributes
         '''
-        meas_grp.attrs.update(image_parms)
         usize, vsize = image.shape
-        meas_grp.attrs['image_size_u'] = usize
-        meas_grp.attrs['image_size_v'] = vsize
-        meas_grp.attrs['translator'] = 'OneView'
-        meas_grp.attrs['num_pixels'] = image.size
-
-        ds_raw_image = VirtualDataset('Raw_Data', np.reshape(image, (-1, 1)))
+        image_parms.attrs['image_size_u'] = usize
+        image_parms.attrs['image_size_v'] = vsize
+        image_parms.attrs['translator'] = 'OneView'
+        image_parms.attrs['num_pixels'] = image.size
+        write_simple_attrs(meas_grp, image_parms)
 
         '''
-        Build Spectroscopic and Position datasets for the image
+        Build Spectroscopic and Position dimensions
         '''
-        spec_desc = Dimension('Intensity', 'a.u.', [1])
-        ds_spec_inds, ds_spec_vals = build_ind_val_dsets(spec_desc, is_spectral=True)
-
+        spec_desc = Dimension('Image', 'a.u.', [1])
         pos_desc = [Dimension('X', 'pixel', np.arange(image.shape[0])),
                     Dimension('Y', 'pixel', np.arange(image.shape[1]))]
-        ds_pos_inds, ds_pos_vals = build_ind_val_dsets(pos_desc, is_spectral=False)
 
-        chan_grp.add_children([ds_raw_image, ds_spec_inds, ds_spec_vals,
-                               ds_pos_inds, ds_pos_vals])
-
-        '''
-        Write the data to file and get the handle for the image dataset
-        '''
-        image_refs = self.hdf.write(root_grp)
-
-        h5_image = get_h5_obj_refs(['Raw_Data'], image_refs)[0]
-
-        '''
-        Link references to raw
-        '''
-        aux_ds_names = ['Position_Indices', 'Position_Values', 'Spectroscopic_Indices', 'Spectroscopic_Values']
-        link_as_main(h5_image, *get_h5_obj_refs(aux_ds_names, image_refs))
+        h5_image = write_main_dataset(chan_grp, np.reshape(image, (-1, 1)), 'Raw_Data',
+                                      'Intensity', 'a.u.',
+                                      pos_desc, spec_desc)
 
         self.root_image_list.append(h5_image)
 
@@ -272,10 +248,10 @@ class OneViewTranslator(Translator):
 
             mean_ronch += image
 
-            self.hdf.flush()
+            self.h5_f.flush()
 
         h5_ronch[:] = mean_ronch / num_files
-        self.hdf.flush()
+        self.h5_f.flush()
 
     def crop_ronc(self, ronc):
         """
@@ -451,53 +427,31 @@ class OneViewTranslator(Translator):
         main_parms.update(image_parms)
 
         # Create the hdf5 data Group
-        root_grp = VirtualGroup('/')
-        root_grp.attrs = root_parms
-        meas_grp = VirtualGroup('Measurement_000')
-        meas_grp.attrs = main_parms
-        chan_grp = VirtualGroup('Channel_000')
+        write_simple_attrs(self.h5_f, root_parms)
+        meas_grp = create_indexed_group(self.h5_f, 'Measurement')
+        write_simple_attrs(meas_grp, main_parms)
+        chan_grp = create_indexed_group(meas_grp, 'Channel')
 
         # Build the Position and Spectroscopic Datasets
-        spec_desc = [Dimension('U', 'pixel', np.arange(usize)), Dimension('V', 'pixel', np.arange(vsize))]
-        ds_spec_ind, ds_spec_vals = build_ind_val_dsets(spec_desc, is_spectral=True)
+        spec_desc = [Dimension('U', 'pixel', np.arange(usize)),
+                     Dimension('V', 'pixel', np.arange(vsize))]
         pos_desc = [Dimension('X', 'pixel', np.arange(scan_size_x)),
                     Dimension('Y', 'pixel', np.arange(scan_size_y))]
-        ds_pos_ind, ds_pos_val = build_ind_val_dsets(pos_desc, is_spectral=False)
 
         ds_chunking = calc_chunks([num_files, num_pixels],
                                   data_type(0).itemsize,
                                   unit_chunks=(1, num_pixels))
 
         # Allocate space for Main_Data and Pixel averaged Data
-        ds_main_data = VirtualDataset('Raw_Data', data=None, maxshape=(num_files, num_pixels),
-                                      chunking=ds_chunking, dtype=data_type, compression='gzip')
-        ds_mean_ronch_data = VirtualDataset('Mean_Ronchigram',
-                                            data=np.zeros(num_pixels, dtype=np.float32),
-                                            dtype=np.float32)
-        ds_mean_spec_data = VirtualDataset('Spectroscopic_Mean',
-                                           data=np.zeros(num_files, dtype=np.float32),
-                                           dtype=np.float32)
-        # Add datasets as children of Measurement_000 data group
-        chan_grp.add_children([ds_main_data, ds_spec_ind, ds_spec_vals, ds_pos_ind,
-                               ds_pos_val, ds_mean_ronch_data, ds_mean_spec_data])
-        meas_grp.add_children([chan_grp])
+        h5_main = write_main_dataset(chan_grp, (num_files, num_pixels), 'Raw_Data',
+                                     'Intensity', 'a.u.',
+                                     pos_desc, spec_desc,
+                                     chunks=ds_chunking, dtype=data_type)
 
-        root_grp.add_children([meas_grp])
-        # print('Writing following tree to this file:')
-        # root_grp.showTree()
+        h5_ronch= chan_grp.create_dataset('Mean_Ronchigram', shape=[num_pixels], dtype=np.float32)
+        h5_mean_spec = chan_grp.create_dataset('Spectroscopic_Mean', shape=[num_files], dtype=np.float32)
 
-        h5_refs = self.hdf.write(root_grp)
-        h5_main = get_h5_obj_refs(['Raw_Data'], h5_refs)[0]
-        h5_ronch = get_h5_obj_refs(['Mean_Ronchigram'], h5_refs)[0]
-        h5_mean_spec = get_h5_obj_refs(['Spectroscopic_Mean'], h5_refs)[0]
-        aux_ds_names = ['Position_Indices',
-                        'Position_Values',
-                        'Spectroscopic_Indices',
-                        'Spectroscopic_Values']
-
-        link_as_main(h5_main, *get_h5_obj_refs(aux_ds_names, h5_refs))
-
-        self.hdf.flush()
+        self.h5_f.flush()
 
         return h5_main, h5_mean_spec, h5_ronch
 
