@@ -16,7 +16,7 @@ from ..core.processing.process import Process
 from .proc_utils import get_component_slice
 from ..core.io.hdf_utils import get_h5_obj_refs, find_results_groups, get_indices_for_region_ref, \
     create_region_reference, copy_attributes, reshape_to_n_dims, get_attr, write_main_dataset, \
-    create_results_group, write_simple_attrs
+    create_results_group, write_simple_attrs, create_indexed_group
 from ..io.hdf_writer import HDFwriter
 from ..core.io.io_utils import get_available_memory, format_time
 from ..core.io.dtype_utils import check_dtype, stack_real_to_target_dtype
@@ -325,25 +325,22 @@ def rebuild_svd(h5_main, components=None, cores=None, max_RAM_mb=1024):
     '''
     Create the Group and dataset to hold the rebuild data
     '''
-    rebuilt_grp = VirtualGroup('Rebuilt_Data_', h5_svd_group.name[1:])
-
-    ds_rebuilt = VirtualDataset('Rebuilt_Data', rebuild,
-                                chunking=h5_main.chunks,
-                                compression=h5_main.compression)
-    rebuilt_grp.add_children([ds_rebuilt])
+    rebuilt_grp = create_indexed_group(h5_svd_group, 'Rebuilt_Data')
+    h5_rebuilt = write_main_dataset(rebuilt_grp, rebuild, 'Rebuilt_Data',
+                                    get_attr(h5_main, 'quantity'), get_attr(h5_main, 'units'),
+                                    None, None,
+                                    h5_pos_inds=h5_main.h5_pos_inds, h5_pos_vals=h5_main.h5_pos_vals,
+                                    h5_spec_inds=h5_main.h5_spec_inds, h5_spec_vals=h5_main.h5_spec_vals,
+                                    chunks=h5_main.chunks, compression=h5_main.compression)
 
     if isinstance(comp_slice, slice):
         rebuilt_grp.attrs['components_used'] = '{}-{}'.format(comp_slice.start, comp_slice.stop)
     else:
         rebuilt_grp.attrs['components_used'] = components
 
-    hdf = HDFwriter(h5_main.file)
-    h5_refs = hdf.write(rebuilt_grp)
-
-    h5_rebuilt = get_h5_obj_refs(['Rebuilt_Data'], h5_refs)[0]
     copy_attributes(h5_main, h5_rebuilt, skip_refs=False)
 
-    hdf.flush()
+    h5_main.file.flush()
 
     print('Done writing reconstructed data to file.')
 
