@@ -85,15 +85,14 @@ def visualize_sho_results(h5_main, save_plots=True, show_plots=True, cmap=None):
     grp_name = '_'.join([grp_name, sho_grp.name.split('/')[-1].split('-')[0], h5_main.name.split('/')[-1]])
 
     try:
-        h5_pos = h5_file[get_attr(h5_main, 'Position_Indices')]
+        h5_pos = h5_main.h5_pos_inds
     except KeyError:
         print('No Position_Indices found as attribute of {}'.format(h5_main.name))
         print('Rows and columns will be calculated from dataset shape.')
         num_rows = int(np.floor((np.sqrt(h5_main.shape[0]))))
         num_cols = int(np.reshape(h5_main, [num_rows, -1, h5_main.shape[1]]).shape[1])
     else:
-        num_rows = len(np.unique(h5_pos[:, 0]))
-        num_cols = len(np.unique(h5_pos[:, 1]))
+        num_rows, num_cols = h5_main.pos_dim_sizes
 
     try:
         h5_spec_vals = h5_file[get_attr(h5_main, 'Spectroscopic_Values')]
@@ -155,10 +154,10 @@ def visualize_sho_results(h5_main, save_plots=True, show_plots=True, cmap=None):
         phase_mat = phase_mat.reshape(num_rows, num_cols)
         rsqr_mat = rsqr_mat.reshape(num_rows, num_cols)
 
-        fig_ms, ax_ms = plot_map_stack(np.dstack((amp_mat, freq_mat, q_mat, phase_mat, rsqr_mat)),
-                                       num_comps=5, color_bar_mode='each', heading=grp_name,
-                                       title=['Amplitude (mV)', 'Frequency (kHz)', 'Quality Factor', 'Phase (deg)',
-                                              'R^2 Criterion'], cmap=cmap)
+        fig_ms, ax_ms = plot_map_stack(np.dstack((amp_mat, freq_mat, q_mat, phase_mat, rsqr_mat)).T,
+                                       num_comps=5, color_bar_mode='each', title=grp_name,
+                                       subtitle=['Amplitude (mV)', 'Frequency (kHz)', 'Quality Factor', 'Phase (deg)',
+                                                 'R^2 Criterion'], cmap=cmap)
 
         fig_list.append(fig_ms)
         if save_plots:
@@ -886,6 +885,8 @@ def jupyter_visualize_parameter_maps(h5_loop_parameters, cmap=None, **kwargs):
     parameter_map_stack = np.reshape(h5_loop_parameters[parameter_names[0]],
                                      [pos_dims[0], pos_dims[1], -1])
 
+    parameter_map_stack = np.moveaxis(parameter_map_stack, -1, 0)
+
     loop_spec_labs = h5_loop_parameters.spec_dim_labels
 
     kwargs.update({'cmap': get_cmap_object(cmap)})
@@ -905,6 +906,8 @@ def jupyter_visualize_parameter_maps(h5_loop_parameters, cmap=None, **kwargs):
     def update_loop_maps(parameter_name):
         parameter_map_stack = np.reshape(h5_loop_parameters[parameter_name],
                                          [pos_dims[0], pos_dims[1], -1])
+        parameter_map_stack = np.moveaxis(parameter_map_stack, -1, 0)
+
         fig.suptitle('Maps of Loop Parameter {}'.format(parameter_name))
         # Loop over all axes
         for icycle, ax_cycle in enumerate(axes[:num_cycles]):
